const fs = require('fs')
const path = require('path')
const readline = require('readline')

const mkdirp = require('mkdirp')
const async = require('async')
const glob = require('glob')

const {compose} = require('@simonjayhawkins/urban-chainsaw')
const {transformJSON} = require('./parse-response')
const COLUMNS = require('./columns')

const scope = path.basename(process.argv[1])
const {timenow} = require('@simonjayhawkins/sturdy-potato')(scope)

const barcodeIsValid = require('@simonjayhawkins/fluffy-barnacle').isValid

const getKeys = array => new Set(array.map(v => v.Id))
const responseIsValid = res => res.Response && res.ResponseHeaderMetadata && res.ResponseHeaderMetadata['x-mws-timestamp']

module.exports =
  class DataService {
    constructor (options) {
      this.eventSourceFilePrefix = options.eventSourceFilePrefix
      this.eventSourceFileExt = options.eventSourceFileExt
      this.eventSourceFileList = []
      this.snapshotFile = `${options.snapshotFilePrefix}.${options.snapshotFileExt}`
      this.mkdirp = mkdirp

      this.data = []
      this.dataObject = { }
      this.index = new Map()

      this.verbose = options.verbose
    }

    init (callback) {
      this.updateSnapshot(err => callback(err))
    }

    save (response, callback) {
      if (!responseIsValid(response)) return callback(new Error('Invalid Response Format'))
      this.processResponse(response, err => callback(err))
    }

    processResponse (response, callback) {
      const date = response.ResponseHeaderMetadata['x-mws-timestamp'].slice(0, 10)
      const logFile = `${this.eventSourceFilePrefix}-${date}.${this.eventSourceFileExt}`
      const header = (this.eventSourceFileList.indexOf(logFile) === -1)

      transformJSON(response, header, (err, data) => {
        if (err) return callback(err)
        this.saveToResponseLog(logFile, data, err => callback(err))
      })
    }

    saveToResponseLog (logFile, data, callback) {
      const _path = path.dirname(logFile)

      this.mkdirp(_path, err => {
        if (err) return callback(err.message)

        fs.appendFile(logFile, data, err => {
          if (err) return callback(err.message)
          if (this.eventSourceFileList.indexOf(logFile) === -1) this.eventSourceFileList.push(logFile)
          callback()
        })
      })
    }

    getStaleKeys (timestamp) {
      return [...new Set(
        this.data
          .filter(v => v.timestamp < timestamp)
          .map(v => v.Id)
      )]
    }

    getValidKeys (idList) {
      const keys = getKeys(this.data)
      if (idList) return idList.filter(v => keys.has(v))
      return [...keys]
    }

    getNonExistentKeys (idList) {
      const keys = getKeys(this.data)
      return idList.filter(v => !keys.has(v))
    }

    saveSnapshot (callback) {
      const filename = this.snapshotFile

      this.data = []
      Object.keys(this.dataObject).forEach(key => this.data.push(this.dataObject[key]))

      const array = this.data.reverse()

      fs.writeFile(
        filename,
        compose(
          array,
          {
            header: true,
            columns: COLUMNS
          }
        ),
        'utf8',
        err => {
          if (this.verbose) timenow(`writing to ${filename}..`)
          if (err) return callback(err.message)
          timenow(`${array.length} barcodes in ${filename}`)
          callback()
        }
      )
    }

    updateSnapshot (callback) {
      this.readLogFile(this.snapshotFile, err => {
        if (err) timenow(err)

        const globPattern = `${this.eventSourceFilePrefix}*.${this.eventSourceFileExt}`

        glob(globPattern, null, (err, files) => {
          if (err) return callback(err)

          if (!files.length) {
            this.noDatabaseMessage = `ENOENT: no such file or directory, open '${globPattern}'`
            this.data = []
            Object.keys(this.dataObject).forEach(key => this.data.push(this.dataObject[key]))
            return callback()
          }

          const snapshotTimestamp = Array.from(this.index.values()).sort().pop()

          if (snapshotTimestamp) {
            const snapshotDate = snapshotTimestamp.slice(0, 10)
            const re = new RegExp(`${this.eventSourceFilePrefix}-(\\d{4}-\\d{2}-\\d{2})\\.${this.eventSourceFileExt}`)
            this.eventSourceFileList = files.filter(file => (file.match(re)[1] >= snapshotDate))
          } else {
            this.eventSourceFileList = files
          }

          async.eachSeries(
            this.eventSourceFileList.sort((a, b) => (a > b) ? -1 : 1),
            this.readLogFile.bind(this),
            err => {
              if (err) return callback(err)
              this.saveSnapshot(err => callback(err))
            }
          )
        })
      })
    }

    readLogFile (file, callback) {
      if (this.verbose) timenow(`reading from ${file}..`)

      let keys = null

      const parseLine = line => {
        if (!line.length) return
        const array = line.split('\t')
        if (keys) {
          const obj = Object.assign({}, ...array.map((f, i) => ({[keys[i]]: f})))

          if (!obj.Id || !obj.timestamp || !barcodeIsValid(obj.Id)) return

          if (!this.index.has(obj.Id) ||
            (this.index.has(obj.Id) && this.index.get(obj.Id) < obj.timestamp)) {
            this.index.set(obj.Id, obj.timestamp)
            this.dataObject[obj.Id] = obj
          }
        } else {
          keys = array
        }
      }

      const input = fs.createReadStream(file, 'utf8')

      input.on('error', err => callback(err))

      readline
        .createInterface({input: input})
        .on('line', parseLine)
        .on('close', callback)
    }
}
