#!/usr/bin/env node

const fs = require('fs')
const path = require('path')
const os = require('os')
const program = require('commander')

const scope = path.basename(process.argv[1])
const {timenow} = require('@simonjayhawkins/sturdy-potato')(scope)

const {getCredentials} = require('@simonjayhawkins/shiny-robot')
const {matchProducts} = require('./match-products')

const {version, description} = require('../package.json')
const config = require('../mpconfig.json')

const exit = err => {
  timenow(err)
  process.exit(1)
}

const configFileDefault = 'mpconfig.json'
const mwsCredentialsFileDefault = `${os.homedir()}/mwsconfig.json`

let filename

program
  .description(description)
  .version(version)
  .arguments('[file]')
  .option('-v, --verbose', 'additional output')
  .option('-c, --check', 'validate barcode check digits and exit')
  .option('-d, --dataservice-config <path>', `set dataservice config path. defaults to ${configFileDefault}`)
  .option('-m, --mws-credentials <path>', `set MWS credentials path. defaults to ${mwsCredentialsFileDefault}`)
  .action(file => {
    filename = file
    fs.readFile(filename, 'utf8', (err, contents) => {
      if (err) return exit(err.message)

      const barcodeList = [
        ...new Set(
          contents
            .split('\n')
            .map(v => v.split('\t')[0].trim())
            .filter(v => /^[0-9]+$/.test(v))
        )
      ]

      if (!barcodeList.length) return exit(`ERROR: check ${file}`)

      timenow(`${program.check ? 'checking' : 'using'} ${file}`)
      main(barcodeList)
    })
  })
  .parse(process.argv)

if (typeof filename === 'undefined') {
  timenow(`${program.check ? 'checking' : 'using'} existing barcode database..`)
  main()
}

function main (barcodeList) {
  const configFile = program.dataserviceConfig || configFileDefault
  const mwsCredentialsFile = program.mwsCredentials || mwsCredentialsFileDefault

  fs.readFile(configFile, 'utf8', (err, contents) => {
    if (err) {
      if (program.dataserviceConfig) return exit(err.message)
      else timenow('WARNING! Using default configuration')
    } else {
      try {
        var _config = JSON.parse(contents)
      } catch (e) {
        exit(`${configFile}: Invalid JSON`)
      }
      Object.assign(config, _config)
    }

    getCredentials(mwsCredentialsFile, (err, credentials) => {
      if (err) return exit(err)

      const dataService = config.dataService.services.filter(service => service.name === config.dataService.default)
      if (!dataService.length) exit(`${config.dataService.default} dataservice not configured`)
      timenow(`using ${config.dataService.default} dataservice`)

      matchProducts(barcodeList, {
        dataService: dataService[0],
        credentials: credentials,
        verbose: program.verbose,
        check: program.check
      }, err => {
        if (err) return exit(err)
        timenow('Done.')
      })
    })
  })
}
