const path = require('path')
const moment = require('moment-timezone')
const async = require('async')

const barcodeIsValid = require('@simonjayhawkins/fluffy-barnacle').isValid
const {Products} = require('@simonjayhawkins/shiny-robot')

const scope = path.basename(process.argv[1])
const {timenow} = require('@simonjayhawkins/sturdy-potato')(scope)

module.exports = {
  matchProducts: matchProducts,
  validateBarcodeList: validateBarcodeList,
  barcodeListChunk: barcodeListChunk
}

function matchProducts (barcodeList, options, callback) {
  const DataService = require(`./${options.dataService.module}`)

  let barcodeListValid = []

  if (barcodeList) {
    barcodeListValid = validateBarcodeList(barcodeList, options.verbose || options.check)
    if (!barcodeListValid.length) return callback(new Error('ERROR: no valid barcodes'))
    if (options.check) return callback()
  }

  Object.assign(options.dataService, {verbose: options.verbose})
  const dataService = new DataService(options.dataService)

  dataService.init(err => {
    if (err) return callback(err)

    if (barcodeList == null) {
      if (!dataService.index.size) return callback(new Error(dataService.noDatabaseMessage || 'no existing barcodes found'))

      if (options.check) {
        barcodeList = dataService.getValidKeys()
        validateBarcodeList(barcodeList, true)
        return callback()
      }

      // get stale barcodes
      const staleTimestamp = moment().subtract(14, 'days').tz('UTC').toISOString()
      barcodeList = dataService.getStaleKeys(staleTimestamp)
      barcodeListValid = validateBarcodeList(barcodeList, options.verbose)
      if (!barcodeListValid.length) {
        timenow('no stale records')
        return callback()
      }
    } else {
      // get new/missing barcodes
      barcodeListValid = dataService.getNonExistentKeys(barcodeListValid)
      if (!barcodeListValid.length) {
        timenow('no new/missing barcodes')
        return callback()
      }
    }

    const products = new Products(options.credentials)

    const _callback = callback
    let first = true

    async.eachSeries(
      barcodeListChunk(barcodeListValid),
      (chunkData, callback) => {
        if (first || options.verbose) {
          timenow(`${chunkData.remaining} barcodes to process`)
          first = false
        }

        if (options.verbose) {
          timenow(`requesting ${JSON.stringify(chunkData.IdList)}`)
        }

        products.getMatchingProductForId(
          {
            IdType: (chunkData.IdList[0].length === 12) ? 'UPC' : 'EAN',
            IdList: chunkData.IdList
          },
          (err, response) => {
            if (err) return callback(err)

            if (options.verbose) {
              const quota = Object.assign({},
                ...Object.keys(response.ResponseHeaderMetadata)
                  .filter(v => v.startsWith('x-mws-quota'))
                  .map(v => ({[v]: response.ResponseHeaderMetadata[v]}))
              )
              timenow(JSON.stringify(quota))
            }

            dataService.save(response, err => (err) ? _callback(err) : void 0)

            setTimeout(callback, 1000)
          })
      },
      err => {
        if (err) return callback(err)
        dataService.updateSnapshot(err => callback(err))
      }
    )
  })
}

function validateBarcodeList (barcodeList, verbose) {
  return barcodeList
    .filter(v => {
      const condition = v.length === 12 || v.length === 13
      if (verbose && !condition) timenow(`${v}: not EAN/UPC barcode`)
      return condition
    })
    .filter(v => {
      const condition = barcodeIsValid(v)
      if (verbose && !condition) timenow(`${v}: bad check digit`)
      return condition
    })
}

function * barcodeListChunk (barcodeList) {
  let array = [...barcodeList]
  while (array.length) {
    const remaining = array.length
    const chunk = [array.shift()]
    let offset = 0
    while (chunk.length < 5 && offset < array.length) {
      if (array[offset].length === chunk[0].length) {
        chunk.push(array[offset])
        array.splice(offset, 1)
      } else {
        offset++
      }
    }
    yield {
      remaining: remaining,
      IdList: chunk
    }
  }
}
