/* global it, describe */

const {assert} = require('chai')
const sinon = require('sinon')
const {validateBarcodeList, barcodeListChunk} = require('../src/match-products')

const mockBarcodeListWithInvalidValues = [
  '120000000005',
  '1300000000000',
  '1300000000001', // invalid check digit
  '14000000000003', // 14 digits
  '11000000006' // 11 digits
]

const mockBarcodeList = [
  '120000000005',
  '120000000012',
  '120000000029',
  '120000000036',
  '120000000043',
  '120000000050',
  '120000000067',
  '120000000074',
  '120000000081',
  '120000000098',
  '1300000000000',
  '1300000000017',
  '1300000000024',
  '1300000000031',
  '1300000000048',
  '1300000000055',
  '1300000000062',
  '1300000000079',
  '1300000000086',
  '1300000000093',
  '1300000000109'
]

const mockBarcodeListReordered = [
  '120000000005',
  '120000000012',
  '120000000029',
  '1300000000000',
  '1300000000017',
  '1300000000024',
  '1300000000031',
  '1300000000048',
  '1300000000055',
  '120000000036',
  '120000000043',
  '120000000050',
  '120000000067',
  '120000000074',
  '120000000081',
  '120000000098',
  '1300000000062',
  '1300000000079',
  '1300000000086',
  '1300000000093',
  '1300000000109'
]

describe('match-products module', function () {
  describe('validateBarcodeList function', function () {
    it('should return an array of valid barcodes', function () {
      const stub = sinon.stub(console, 'warn').returns(void 0)
      assert.deepEqual(validateBarcodeList(mockBarcodeListWithInvalidValues, false), mockBarcodeListWithInvalidValues.slice(0, 2))
      assert.deepEqual(validateBarcodeList(mockBarcodeList, false), mockBarcodeList)
      assert.deepEqual(validateBarcodeList(mockBarcodeListReordered, false), mockBarcodeListReordered)
      assert.deepEqual(validateBarcodeList([], false), [])
      sinon.assert.notCalled(stub)
      stub.restore()
    })

    it('should print invalid barcodes to log when verbose function parameter is true', function () {
      const stub = sinon.stub(console, 'warn').returns(void 0)
      assert.deepEqual(validateBarcodeList(mockBarcodeListWithInvalidValues, true), mockBarcodeListWithInvalidValues.slice(0, 2))
      assert.deepEqual(validateBarcodeList(mockBarcodeList, true), mockBarcodeList)
      assert.deepEqual(validateBarcodeList(mockBarcodeListReordered, true), mockBarcodeListReordered)
      assert.deepEqual(validateBarcodeList([], true), [])
      sinon.assert.callCount(stub, 3)
      stub.restore()
    })
  })

  describe('barcodeListChunk function', function () {
    const [...chunkData] = barcodeListChunk(mockBarcodeListReordered)

    const results = chunkData.map(v => v.IdList)

    it('should return a non-empty array with no more than 5 elements', function () {
      results.forEach(chunk => {
        assert.isArray(chunk)
        assert.isAtLeast(chunk.length, 1)
        assert.isAtMost(chunk.length, 5)
      })
    })

    it('should return an array with all elements having the same number of characters', function () {
      results.forEach(chunk => {
        const length = chunk[0].length
        chunk.forEach(element => {
          assert.lengthOf(element, length)
        })
      })
    })

    it('should return an array with each element retaining the ordering of the source array', function () {
      results.forEach(chunk => {
        for (let i = 0; i < chunk.length - 1; i++) {
          const [a, b] = chunk.slice(i)
          assert.isBelow(mockBarcodeListReordered.indexOf(a), mockBarcodeListReordered.indexOf(b))
        }
      })
    })

    it('should return array with the first element of each chunk retaining the ordering of the source array', function () {
      for (let i = 0; i < results.length - 1; i++) {
        const [a, b] = results.slice(i)
        assert.isBelow(mockBarcodeListReordered.indexOf(a[0]), mockBarcodeListReordered.indexOf(b[0]))
      }
    })
  })
})
