/* global it, describe */

const assert = require('chai').assert
const parseResponse = require('../src/parse-response')
const fs = require('fs')

const mockJSON = JSON.parse(fs.readFileSync('test/fixtures/1489518b-8c37-44f7-ae53-EXAMPLEd89db0.json', 'utf8'))

describe('parse-response module', function () {
  describe('parseXML method', function () {
    describe('using test fixture GetMatchingProductForIdResponse.xml', function () {
      it('should convert xml response into array of JavaScript objects', function (done) {
        fs.readFile('./test/fixtures/GetMatchingProductForIdResponse.xml', 'utf8', function (err, contents) {
          if (err) done(err)
          else {
            parseResponse.parseXML(contents, function (err, data) {
              if (err) done(err)
              else {
                assert.isArray(data)
                data.forEach(value => {
                  assert.isObject(value)
                  assert.property(value, 'Id')
                  assert.property(value, 'status')
                  assert.property(value, 'RequestId')
                })

                const expected = [
                  {
                    ASIN: '1933988665',
                    CurrencyCode: 'USD',
                    Id: '9781933988665',
                    ListPrice: '44.99',
                    ProductCategoryId: 'book_display_on_website',
                    Rank: '59485',
                    RequestId: '7ba3245e-a213-430a-bea9-EXAMPLE38d76',
                    Title: 'Algorithms of the Intelligent Web',
                    status: 'Success'
                  },
                  {
                    ASIN: '059035342X',
                    CurrencyCode: 'USD',
                    Id: '0439708184',
                    ListPrice: '10.99',
                    ProductCategoryId: 'book_display_on_website',
                    Rank: '362',
                    RequestId: '7ba3245e-a213-430a-bea9-EXAMPLE38d76',
                    Title: "Harry Potter and the Sorcerer's Stone (Book 1)",
                    status: 'Success'
                  },
                  {
                    ASIN: '0439708184',
                    CurrencyCode: undefined,
                    Id: '0439708184',
                    ListPrice: undefined,
                    ProductCategoryId: 'book_display_on_website',
                    Rank: '2091024',
                    RequestId: '7ba3245e-a213-430a-bea9-EXAMPLE38d76',
                    Title: "Harry Potter and the Sorcerer's Stone",
                    status: 'Success'
                  }
                ]

                assert.deepEqual(data, expected)

                done()
              }
            })
          }
        })
      })
    })

    describe('using test fixture GetMatchingProductForIdResponseWithError.xml', function () {
      it('should convert xml response into array of JavaScript objects', function (done) {
        fs.readFile('./test/fixtures/GetMatchingProductForIdResponseWithError.xml', 'utf8', function (err, contents) {
          if (err) done(err)
          else {
            parseResponse.parseXML(contents, function (err, data) {
              if (err) done(err)
              else {
                assert.isArray(data)
                data.forEach(value => {
                  assert.isObject(value)
                  assert.property(value, 'Id')
                  assert.property(value, 'status')
                  assert.property(value, 'RequestId')
                })

                const expected = [
                  {
                    ErrorMessage: 'Invalid ISBN identifier 0439708623 for marketplace ATVPDKIKX0DER',
                    Id: '0439708623',
                    RequestId: 'a34b6cca-d7dc-4939-b226-b13ea3ac1d88',
                    status: 'ClientError'
                  },
                  {
                    ASIN: '1933988665',
                    CurrencyCode: 'USD',
                    Id: '9781933988665',
                    ListPrice: '44.99',
                    ProductCategoryId: 'book_display_on_website',
                    Rank: '689405',
                    RequestId: 'a34b6cca-d7dc-4939-b226-b13ea3ac1d88',
                    Title: 'Algorithms of the Intelligent Web',
                    status: 'Success'
                  }
                ]

                assert.deepEqual(data, expected)

                done()
              }
            })
          }
        })
      })
    })
  })

  describe('parseJSON method', function () {
    describe('using test fixture GetMatchingProductForIdResponse.xml', function () {
      it('should convert json response object into array of JavaScript objects', function (done) {
        fs.readFile('./test/fixtures/GetMatchingProductForIdResponse.xml', 'utf8', function (err, contents) {
          if (err) done(err)
          else {
            const mockJSON = {
              'Response': contents,
              'ResponseHeaderMetadata': {
                'x-mws-request-id': 'string',
                'x-mws-response-context': 'string',
                'x-mws-timestamp': '2016-09-09T20:57:30.057Z',
                'x-mws-quota-max': '34480.0',
                'x-mws-quota-remaining': '34480.0',
                'x-mws-quota-resetsOn': '2016-09-09T21:54:00.000Z'
              }
            }

            parseResponse.parseJSON(mockJSON, function (err, data) {
              if (err) done(err)
              else {
                assert.isArray(data)
                data.forEach(value => {
                  assert.isObject(value)
                  assert.property(value, 'Id')
                  assert.property(value, 'status')
                  assert.property(value, 'RequestId')
                  assert.property(value, 'timestamp')
                  assert.strictEqual(value.timestamp, '2016-09-09T20:57:30.057Z')
                })

                done()
              }
            })
          }
        })
      })
    })
  })

  describe('transformJSON method', function () {
    describe('using test fixture 1489518b-8c37-44f7-ae53-EXAMPLEd89db0.json', function () {
      it('should produce tab-separated output', function (done) {
        parseResponse.transformJSON(mockJSON, false,
          function (err, data) {
            if (err) done(err)

            assert.isString(data)

            assert.strictEqual(data.trim().split('\n').length, 7)

            data.trim().split('\n').forEach(value => {
              assert.strictEqual(value.split('\t').length, 10)
              assert.strictEqual(value.split('\t')[9], '2016-09-11T16:23:31Z')
            })

            let expected = [
              '5026619991411',
              'Success',
              null,
              'B013R7UC5K',
              'Inflatable Pirate Sword, Inflatable Parrot & Inflatable Crutch (One each)',
              null,
              null,
              'toy_display_on_website',
              '578129',
              '2016-09-11T16:23:31Z'
            ].join('\t')

            assert.strictEqual(data.trim().split('\n')[0], expected)

            expected = [
              '5051090016674',
              'Success',
              null,
              'B0015Q07X8',
              'Bristol Novelty Penguin Adult Costume with Lightweight Bow Tie',
              '14.99',
              'GBP',
              'toy_display_on_website',
              '37690',
              '2016-09-11T16:23:31Z'
            ].join('\t')

            assert.strictEqual(data.trim().split('\n')[3], expected)

            done()
          })
      })
    })
  })
})
